import 'package:test/test.dart';
import 'package:fft/fft.dart';
//import 'package:complex/complex.dart';
import 'dart:math' as math;

void main() {
  test("foobar works", () {
    expect(true, equals(true));
  });

  test("complexpolar works", () {
    var cp = new Complex.polar(2,0);
    expect(cp.real, equals(2));
  });

  var cp1 = new Complex.cartesian(1,0);
  var cp2 = new Complex.cartesian(0,1);
    
  test("complexpolar adding", () {
    var cp3 = cp1 + cp2;
    expect(cp3.argument, equals(math.PI/4));
  });

  test("complex conjugate", () {
    var cp1 = new Complex.cartesian(2.1, 1.7);
    var cp1c = new Complex.cartesian(2.1, -1.7);
    expect(cp1.conjugate(), equals(cp1c));
    expect(new Complex.polar(1, math.PI/2).conjugate(), equals(new Complex.cartesian(0,-1)));
  });
  
  test("complexpolar conversion", () {
    var cp1 = new Complex.cartesian(1,0);
    var cp2 = new Complex.cartesian(0,1);
    var cp3 = new Complex.cartesian(-1,0);
    var cp4 = new Complex.cartesian(0,-1);
    expect(cp1.argument, equals(0));
    expect(cp2.argument, equals(math.PI/2));
    expect(cp3.argument, equals(math.PI));
    expect(cp4.argument, equals(-math.PI/2));
  });

  test("complexpolar operators", () {
    var cp1 = new Complex.cartesian(1,1) + new Complex.polar(5, math.atan(3/4));
    expect(cp1.real, equals(5));
    expect(cp1.imaginary, equals(4));
    var cp2 = new Complex.polar(2, math.PI * 3/4) * new Complex.polar(-2, -math.PI/4);
    expect(cp2.real, equals(0));
    expect(cp2.imaginary, equals(4));
    expect(new Complex.polar(2, math.PI/2)==new Complex.cartesian(0,2), equals(true));

    expect(-new Complex.cartesian(1,1)==newComplex.cartesian(-1,-1), equals(true));
  });
    
  
  test("complexpolar adding timed", () {
    Stopwatch s = new Stopwatch();
    var cp4 = new Complex.polar(1,1);
    var cp3 = new Complex.polar(1,0);
    num reps = math.pow(10,6);
    s.start();
    for(int i=0; i<reps; i++) {
      cp3 = cp3+cp4;
    }
    s.stop();
    print("$reps repetitions: ${s.elapsedMilliseconds}ms");
  });

  
  test("complexpolar multiplication timed", () {
    Stopwatch s = new Stopwatch();
    var cp4 = new Complex.polar(1,1);
    var cp3 = new Complex.polar(1,0);
    num reps = math.pow(10,6);
    s.start();
    for(int i=0; i<reps; i++) {
      cp3 = cp3*cp4;
    }
    s.stop();
    print("$reps repetitions: ${s.elapsedMilliseconds}ms");
  });


  test("complexpolar imaginary timed", () {
    Stopwatch s = new Stopwatch();
    var cp4 = new Complex.polar(1,1);
    double total = 0.0;
    num reps = math.pow(10,6);
    s.start();
    for(int i=0; i<reps; i++) {
      total += cp4.imaginary;
      cp4 = cp4.turn(0.1);
    }
    s.stop();
    print("$reps repetitions: ${s.elapsedMilliseconds}ms");
  });
  
  test("complexpolar invert timed", () {
    Stopwatch s = new Stopwatch();
    var cp4 = new Complex.polar(1,1);
    num reps = math.pow(10,6);
    s.start();
    for(int i=0; i<reps; i++) {
      cp4=cp4.inverse();
    }
    s.stop();
    print("$reps repetitions: ${s.elapsedMilliseconds}ms");
  });

  
  
  test("complexpolar real", () {
    expect(cp1.real, equals(1));
    var cp3 = new Complex.polar(-2,math.PI);
    expect(cp3.real, equals(2));
  });

  test("complexpolar imaginary", () {
    expect(cp1.imaginary, equals(0));
    var cp3 = new Complex.polar(-2, math.PI * 3/4);
    num res = cp3.imaginary + 2* math.SQRT1_2;
    bool passed = res<0.0001 && res>-0.0001;
    expect(passed, equals(true));
  });

  test("complexpolar multiply", () {
    var cp3 = cp1 * cp2;
    expect(cp3.argument, equals(math.PI/2));
  });

  test("complexpolar turn", () {
    var cp3 = cp1.turn(math.PI/2);
    expect(cp3.argument, equals(math.PI/2));
    expect(cp3.modulus, equals(1.0));
  });

  test("complexpolar pow", () {
    var cp3 = new Complex.polar(2,math.PI/2);
    var cp4 = cp3.pow(3);
    expect(cp4.modulus, equals(8));
    expect(cp4.argument, equals( 3 * math.PI/2));
  });

  test("complexpolar stretch", () {
    expect(cp2.stretch(4).modulus, equals(4));
  });


  test("complexpolar invert", () {
    var cp3 = new Complex.polar(2,math.PI/4);
    var cp4 = cp3.inverse();
    expect(cp4.modulus, equals(1/2));
    expect(cp4.argument, equals(7*math.PI/4));
  });

  var complexPolarList1 = [ new Complex.polar(1,0), new Complex.polar(-1,0)];
  var complexPolarList2 = [ new Complex.polar(0,1), new Complex.polar(0,-1)];
  
  test("SplitList runs", () {
    var sl = new SplitList<Complex>(complexPolarList1, complexPolarList2);
    expect(true, equals(true));
  });

  var sln = new SplitList<num>.fromList([1,2,3,4]);
  test("SplitList splits", () {
    expect(sln.evens[0], equals(1));
    expect(sln.odds[1], equals(4));
  });
  
  test("SplitList zips", () {
    expect(sln.zip()[2], equals(3));
  });

  test("FFT runs", () {
    var fft = new FFT().Transform([1.0,0.0,-1.0, 0.0]);
    //    print(fft);
    expect(fft.length, equals(4));
  });

  test("Hamming works", () {
    var l = [1,1,1,1,1,1,1,1,1];
    var h = Window.Hamming(l);
    //    print(h);
    expect(h[(l.length-1)~/2], equals(1));
  });

  test("Hann works", () {
    var l = [1,1,1,1,1,1,1,1,1];
    var h = Window.Hann(l);
    //    print(h);
    expect(h[(l.length-1)~/2], equals(1));
  });

  test("fft does something that makes sense", () {
    int l2len = 12;
    int len = math.pow(2, l2len);
    var frequencies = [2,5,15,35];
    var input = new List<num>.filled(len,0);
    
    for (int i=0; i<len; i++) {
      frequencies.forEach((int freq) {
        input[i] += math.cos(2*math.PI * i * freq / len);
      });
    }
    var fft = new FFT().Transform(Window.Hamming(input));
    var results = new List<num>();
    frequencies.forEach((int i)=>results.add(fft[i].modulus));
    var res = results.fold(true, (bool val, double n)=>val && (n>5.0));
    expect(res, equals(true)); 

  });
  test("fft has right sign for phase", () {
    int l2len = 8;
    int len = math.pow(2, l2len);
    var input = new List<num>.filled(len,0);
    
    for (int i=0; i<len; i++) {
      input[i] += math.sin(2*math.PI * i * 4/len);
      input[i] += math.cos(2*math.PI * i * 7/len);
    }
    var fft = new FFT().Transform(Window.Hamming(input));
    var sinArgument = fft[4].argument;
    var cosArgument = fft[7].argument;
    expect(sinArgument<(math.PI * -0.45) && sinArgument>(math.PI * -0.55), equals(true));
    expect(cosArgument<0.05 && cosArgument>-0.05, equals(true));
  });

  test("fft produces correct results", () {
    var vals = [1,2,3,1,2,3,1,2];
    var fft = new FFT().Transform(vals);
    print(fft);

  });
}
